pub type Id = u64;
pub type FileSize = u64;
pub type ImageDimension = u64;

#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
pub struct Posts {
    #[serde(alias = "posts")]
    inner: Vec<Post>,
}

impl Into<Vec<Post>> for Posts {
    fn into(self) -> Vec<Post> {
        self.inner
    }
}

#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
pub struct Post {
    pub id: Id,
    pub created_at: String,
    pub updated_at: String,
    pub tags: PostTags,
    pub file: PostFile,
    pub rating: String,
    pub fav_count: u16,
    pub sources: Vec<String>,
    pub approver_id: Option<Id>,
    pub uploader_id: Id,
    pub description: String,
    pub comment_count: u16,
    pub has_notes: bool,
    pub is_favorited: bool,
    pub change_seq: Option<u128>,
    pub relationships: PostRelationships,
}

#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
pub struct PostTags {
    pub artist: Vec<String>,
    pub character: Vec<String>,
    pub copyright: Vec<String>,
    pub general: Vec<String>,
    pub invalid: Vec<String>,
    pub lore: Vec<String>,
    pub meta: Vec<String>,
    pub species: Vec<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
pub struct PostFile {
    pub width: ImageDimension,
    pub height: ImageDimension,
    pub ext: String,
    pub size: FileSize,
    pub md5: String,
    pub url: String,
}

#[derive(serde::Serialize, serde::Deserialize, Clone, Debug)]
pub struct PostRelationships {
    pub parent_id: Option<Id>,
    pub has_children: bool,
    pub has_active_children: bool,
}
